#include <File.au3>
#include <GDIPlus.au3>
#include <GDIPlusConstants.au3>
#include <WinAPI.au3>

Global $hDll = DllOpen("FreeImage.dll")
Global $hDll_1 = DllOpen("FreeImage_new.dll")

_GDIPlus_Startup()

Global $tGUID = _WinAPI_GUIDFromString("{557CF401-1A04-11D3-9A73-0000F81EF32E}")
Global $pGUID = DllStructGetPtr($tGUID)

Global $tParams = _GDIPlus_ParamInit(1)
Global $pParams = DllStructGetPtr($tParams)
Global $tData = DllStructCreate("int Quality")
DllStructSetData($tData, "Quality", 50)
_GDIPlus_ParamAdd($tParams, $GDIP_EPGQUALITY, 1, $GDIP_EPTLONG, DllStructGetPtr($tData))

Global $aFiles = _FileListToArray(@ScriptDir & "\BMP\", "*.bmp")
;$aFiles[0] = 20

Global $aPID[10], $bRunning
$iTimer = TimerInit()
For $i = 1 To $aFiles[0] Step 9
	For $j = 0 To 9
		If $i + $j > $aFiles[0] Then ExitLoop
		$aPID[$j] = _ConvertBMP2JPG_ImageMagic(@ScriptDir & "\BMP\" & $aFiles[$i + $j])
	Next
	Do
		$bRunning = False
		For $j = 0 To 9
			If ProcessExists($aPID[$j]) Then $bRunning = True
		Next
	Until Not $bRunning
Next
ConsoleWrite("ImageMagick: " & TimerDiff($iTimer) & @CRLF)


Global $iTimer = TimerInit()
For $i = 1 To $aFiles[0]
	_ConvertBMP2JPG(@ScriptDir & "\BMP\" & $aFiles[$i])
Next
ConsoleWrite("GDIPlus: " & TimerDiff($iTimer) & @CRLF)


$iTimer = TimerInit()
For $i = 1 To $aFiles[0]
	Convert2JPG(@ScriptDir & "\BMP\" & $aFiles[$i], StringTrimRight(@ScriptDir & "\BMP\" & $aFiles[$i], 3) & "jpg")
Next
ConsoleWrite("UEZ: " & TimerDiff($iTimer) & @CRLF)


$iTimer = TimerInit()
For $i = 1 To $aFiles[0]
	_ConvertBMP2JPG_FreeImage(@ScriptDir & "\BMP\" & $aFiles[$i])
Next
ConsoleWrite("FreeImage: " & TimerDiff($iTimer) & @CRLF)


$iTimer = TimerInit()
For $i = 1 To $aFiles[0]
	_ConvertBMP2JPG_FreeImage_New(@ScriptDir & "\BMP\" & $aFiles[$i])
Next
ConsoleWrite("FreeImage New: " & TimerDiff($iTimer) & @CRLF)


_GDIPlus_Shutdown()
DllClose($hDll)
DllClose($hDll_1)


Func _ConvertBMP2JPG($sFileName)
	Local $aResult = DllCall($ghGDIPDll, "int", "GdipLoadImageFromFile", "wstr", $sFileName, "ptr*", 0)
	DllCall($ghGDIPDll, "int", "GdipSaveImageToFile", "handle", $aResult[2], "wstr", StringTrimRight($sFileName, 3) & "jpg", "ptr", $pGUID, "ptr", $pParams)
	DllCall($ghGDIPDll, "int", "GdipDisposeImage", "handle", $aResult[2])
EndFunc   ;==>_ConvertBMP2JPG


Func _ConvertBMP2JPG_FreeImage($sFileName)
	Local $aResult = DllCall($hDll, "ptr", "_FreeImage_Load@12", "int", 0, "str", $sFileName, "int", 0)
	DllCall($hDll, "int", "_FreeImage_Save@16", "int", 2, "ptr", $aResult[0], "str", StringTrimRight($sFileName, 3) & "jpg", "int", 50)
	DllCall($hDll, "none", "_FreeImage_Unload@4", "ptr", $aResult[0])
EndFunc   ;==>_ConvertBMP2JPG_FreeImage


Func _ConvertBMP2JPG_FreeImage_New($sFileName)
	Local $aResult = DllCall($hDll_1, "ptr", "_FreeImage_Load@12", "int", 0, "str", $sFileName, "int", 0)
	DllCall($hDll_1, "int", "_FreeImage_Save@16", "int", 2, "ptr", $aResult[0], "str", StringTrimRight($sFileName, 3) & "jpg", "int", 50)
	DllCall($hDll_1, "none", "_FreeImage_Unload@4", "ptr", $aResult[0])
EndFunc   ;==>_ConvertBMP2JPG_FreeImage_New


Func _ConvertBMP2JPG_ImageMagic($sFileName)
	Return Run(@ScriptDir & "\convert.exe " & $sFileName & " " & StringTrimRight($sFileName, 3) & "jpg", @ScriptDir, @SW_HIDE)
EndFunc   ;==>_ConvertBMP2JPG_ImageMagic


; #FUNCTION# ======================================================================================
; Name ..........:         Convert2JPG()
; Description ...:         Converts any image to a JPG file
; Syntax ........:         Convert2JPG($inFile, $outFile[, $scaleW = 1[, $scaleH = 1[, $jpgQuality = 90[, $interpolationMode = 7]]]])
; Parameters ....:         $inFile                        - input file name - should be any image file except jpg
;                                 $outFile                   - output filename
;                                  $scaleW                    - [optional] any number from 0 to x to scale width factor. Default = 1 - no scale (default:1)
;                                  $scaleH                    - [optional]  (default:1)
;                                  $jpgQuality                - [optional] jpg quality settings: 0 - 100 whereas 100 is best quality (default:90)
;                                  $interpolationMode    - [optional] any number from 0-7 (default:7):
;                                                                      0 - Default interpolation mode
;                                                                      1 - Low-quality mode
;                                                                      2 - High-quality mode
;                                                                      3 - Bilinear interpolation. No prefiltering is done
;                                                                      4 - Bicubic interpolation. No prefiltering is done
;                                                                      5 - Nearest-neighbor interpolation
;                                                                      6 - High-quality, bilinear interpolation. Prefiltering is performed to ensure high-quality shrinking
;                                                                      7 - High-quality, bicubic interpolation. Prefiltering is performed to ensure high-quality shrinking
; Return values .:     Success -> True
;                                  Failure:
;                                1 - either no input or output file is given
;                                2 - scale factor width or height is incorrect
;                                3 - jpg quality value is out of range (0-100)
;                                4 - interpolation mode value is out of range (0-7)
;                                5 - $scaleW or $scaleH or $jpgQuality or $interpolationMode values are not numbers
;                                6 - bitmap could not be created
;                                7 - scaled bitmap could not be created
;                                8 - scaled bitmap could not be saved
;                                9 - bitmap could not be created
; Author ........:         UEZ 2011
; Version .......:         v0.90 build 2011-08-14
; Modified ......:
; Remarks .......:
; Related .......:            GDIPlus.au3
; =================================================================================================
Func Convert2JPG($inFile, $outFile, $scaleW = 1, $scaleH = 1, $jpgQuality = 90, $interpolationMode = 7)
	If $inFile = "" Or $outFile = "" Then Return SetError(1, 0, 0)
	If $scaleW < 0 Or $scaleH < 0 Then Return SetError(2, 0, 0)
	If $jpgQuality < 0 Or $jpgQuality > 100 Then Return SetError(3, 0, 0)
	If $interpolationMode < 0 Or $interpolationMode > 7 Then Return SetError(4, 0, 0)
	If Not IsNumber($scaleW) Or Not IsNumber($scaleH) Or Not IsNumber($jpgQuality) Or Not IsNumber($interpolationMode) Then Return SetError(5, 0, 0)
	Local $declared = True
	If Not $ghGDIPDll Then
		_GDIPlus_Startup()
		$declared = False
	EndIf
	If StringRight($outFile, 4) <> ".jpg" Then $outFile &= ".jpg"
	Local Const $sCLSID = _GDIPlus_EncodersGetCLSID("JPG")
	Local $tParams = _GDIPlus_ParamInit(1)
	Local $tData = DllStructCreate("int Quality")
	DllStructSetData($tData, "Quality", $jpgQuality)
	Local $pData = DllStructGetPtr($tData)
	_GDIPlus_ParamAdd($tParams, $GDIP_EPGQUALITY, 1, $GDIP_EPTLONG, $pData)
	Local Const $pParams = DllStructGetPtr($tParams)
	Local Const $hBitmap = _GDIPlus_BitmapCreateFromFile($inFile)
	If Not $hBitmap Then Return SetError(6, 0, 0)
	Local $iW = _GDIPlus_ImageGetWidth($hBitmap)
	Local $iH = _GDIPlus_ImageGetHeight($hBitmap)
	Local $save, $err = 0
	If $scaleW <> 1 Or $scaleH <> 1 Then
		Local Const $newW = $iW * $scaleW
		Local Const $newH = $iH * $scaleH
		Local $aResult = DllCall($ghGDIPDll, "uint", "GdipCreateBitmapFromScan0", "int", $newW, "int", $newH, "int", 0, "int", 0x0026200A, "ptr", 0, "int*", 0)
		If @error Then Return SetError(7, 0, 0)
		Local $hBmp_scaled = $aResult[6]
		Local $hContext = _GDIPlus_ImageGetGraphicsContext($hBmp_scaled)
		DllCall($ghGDIPDll, "uint", "GdipSetInterpolationMode", "handle", $hContext, "int", $interpolationMode)
		_GDIPlus_GraphicsDrawImageRect($hContext, $hBitmap, 0, 0, $newW, $newH)
		_GDIPlus_GraphicsDispose($hContext)
		$save = _GDIPlus_ImageSaveToFileEx($hBmp_scaled, $outFile, $sCLSID, $pParams)
		$tParams = 0
		$tData = 0
		If Not $save Then $err = 8
		_GDIPlus_BitmapDispose($hBitmap)
		_GDIPlus_BitmapDispose($hBmp_scaled)
		If Not $declared Then _GDIPlus_Shutdown()
		If $err Then Return SetError($err, 0, 0)
		Return True
	EndIf
	$save = _GDIPlus_ImageSaveToFileEx($hBitmap, $outFile, $sCLSID, $pParams)
	$tParams = 0
	$tData = 0
	If Not $save Then $err = 9
	_GDIPlus_BitmapDispose($hBitmap)
	If Not $declared Then _GDIPlus_Shutdown()
	If $err Then Return SetError($err, 0, 0)
	Return True
EndFunc   ;==>Convert2JPG